import argparse
import os
import json
import ROOT
import random
import numpy as np
import pandas as pd
from ROOT import gROOT, gInterpreter


def get_hist_contents2(root_hist):
    cfunc = np.vectorize(root_hist.GetBinContent, otypes=[float])
    xfunc = np.vectorize(root_hist.GetXaxis().GetBinCenter, otypes=[float])
    yfunc = np.vectorize(root_hist.GetYaxis().GetBinCenter, otypes=[float])
    xbins = np.arange(1, root_hist.GetNbinsX() + 1)
    ybins = np.arange(1, root_hist.GetNbinsY() + 1)
    xbins, ybins = np.meshgrid(xbins, ybins)
    return xfunc(xbins), yfunc(ybins), cfunc(xbins, ybins)


if __name__ == "__main__":
    # batch mode
    gROOT.SetBatch(True)

    # make sure the relative path for root loading is correct
    owd = os.getcwd()
    script_dir = os.path.dirname(os.path.realpath(__file__))
    os.chdir(os.path.join(script_dir, '..'))
    print('working directory is ' + os.getcwd())

    # argument parser
    parser = argparse.ArgumentParser('extract simulation binned data')
    parser.add_argument('csv2d', help='path to save 2d histogram data')
    args = parser.parse_args()

    # root macros
    gROOT.Macro('rootlogon.C')
    # simulation output from Zhiwen
    sim_dir = script_dir
    sim = [
        ('pmts', 'npe_nsensor_hgc_after_pmt.C', 'npe_nsensor_hgc_after_pmt'),
        ('pmts_nocut', 'npe_nsensor_hgc_before_pmt.C', 'npe_nsensor_hgc_before_pmt'),
        ('quads', 'npe_nsensor_hgc_after_quad.C', 'npe_nsensor_hgc_after_quad'),
        ('quads_nocut', 'npe_nsensor_hgc_before_quad.C', 'npe_nsensor_hgc_before_quad'),
    ]

    df = pd.DataFrame()
    for name, script, fn in sim:
        func_code = open(os.path.join(sim_dir, script)).read()
        gInterpreter.Declare(func_code)
        hist = getattr(ROOT, fn)()
        # hist data
        x, y, counts = get_hist_contents2(hist)
        tmp = pd.DataFrame(columns=['npe', 'nch', 'counts'], data=np.vstack([x.flatten(), y.flatten(), counts.flatten()]).T)
        tmp.loc[:, 'type'] = name.lower()
        df = pd.concat([df, tmp])

    df.to_csv(args.csv2d, index=False)
